require 'erb'
require 'yaml'
require 'columbus3/renderer/sanitizer'

module Columbus3
  # renderer controls the transformation of a V900 tracks into a displayable format,
  # whatever this means
  module LeafletRenderer
    OUTPUT_FILENAME = "_show.html"

    # apply the ERB in html/show.html.erb (which includes all the tracks to display)
    # and save it in the current directory, under _show.html
    def self.show hash
      filename = hash[:filename] || OUTPUT_FILENAME
      files = hash[:files] || []
      
      template = File.join(File.dirname(__FILE__), "/../../html/show.html.erb")
      renderer = ERB.new(File.read(template))
      
      # context: location of css and js + list of files to show
      bower_dir = File.join(File.dirname(__FILE__), "/../../../bower_components")
      @leaflet_providers = File.join(bower_dir, 'leaflet-providers/leaflet-providers.js')
      @leaflet_js = File.join(bower_dir, 'leaflet/dist/leaflet.js')
      @leaflet_css = File.join(bower_dir, 'leaflet/dist/leaflet.css')
      @files = files

      # generate and save to file
      File.open(filename, "w") do |file|
        file << renderer.result(binding)
      end
    end

    # make a v900 track into a geojson file 
    def self.to_javascript_file filename, force = false
      target = to_javascript_filename filename

      if force or not File.exists? target
        # load the template
        template = File.join(File.dirname(__FILE__), "/../../html/track.js.erb")
        renderer = ERB.new(File.read(template))
      
        # context: the track and the track's id
        @track = V900Track.new filename: filename
        @filename = filename

        # generate and save it to file
        File.open(target, "w") do |file|
          file << renderer.result(binding)
        end
      end

    end

    def self.to_javascript_filename filename
      filename + ".js"
    end

    def self.to_leaflet_layername filename
      to_javascript_varname "layer_", Sanitizer::sanitize(filename)
    end
    
    def self.to_javascript_varname prefix, filename
      prefix + Sanitizer::sanitize(filename)
    end
  end
end
